package org.optaweb.vehiclerouting.plugin.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.List;

import org.assertj.core.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.optaweb.vehiclerouting.domain.Coordinates;
import org.optaweb.vehiclerouting.domain.Location;
import org.optaweb.vehiclerouting.domain.RoutingProblem;
import org.optaweb.vehiclerouting.domain.Vehicle;
import org.optaweb.vehiclerouting.domain.VehicleFactory;
import org.optaweb.vehiclerouting.plugin.rest.model.PortableCoordinates;
import org.optaweb.vehiclerouting.plugin.rest.model.RoutingProblemInfo;
import org.optaweb.vehiclerouting.plugin.rest.model.ServerInfo;
import org.optaweb.vehiclerouting.service.demo.DemoService;
import org.optaweb.vehiclerouting.service.region.BoundingBox;
import org.optaweb.vehiclerouting.service.region.RegionService;

@ExtendWith(MockitoExtension.class)
class ServerInfoResourceTest {

    @Mock
    private RegionService regionService;
    @Mock
    private DemoService demoService;
    @InjectMocks
    private ServerInfoResource serverInfoResource;

    @Test
    void serverInfo() {
        // arrange
        List<String> countryCodes = Arrays.asList("XY", "WZ");
        when(regionService.countryCodes()).thenReturn(countryCodes);

        Coordinates southWest = Coordinates.of(-1.0, -2.0);
        Coordinates northEast = Coordinates.of(1.0, 2.0);
        BoundingBox boundingBox = new BoundingBox(southWest, northEast);
        when(regionService.boundingBox()).thenReturn(boundingBox);

        Location depot = new Location(1, Coordinates.of(1.0, 7), "Depot");
        List<Location> visits = Arrays.asList(new Location(2, Coordinates.of(2.0, 9), "Visit"));
        List<Vehicle> vehicles = Arrays.asList(VehicleFactory.testVehicle(1));
        String demoName = "Testing problem";
        RoutingProblem routingProblem = new RoutingProblem(demoName, vehicles, depot, visits);
        when(demoService.demos()).thenReturn(Arrays.asList(routingProblem));

        // act
        ServerInfo serverInfo = serverInfoResource.serverInfo();

        // assert
        assertThat(serverInfo.getCountryCodes()).isEqualTo(countryCodes);
        assertThat(serverInfo.getBoundingBox()).containsExactly(
                PortableCoordinates.fromCoordinates(southWest),
                PortableCoordinates.fromCoordinates(northEast));
        List<RoutingProblemInfo> demos = serverInfo.getDemos();
        Assertions.assertThat(demos).hasSize(1);
        RoutingProblemInfo demo = demos.get(0);
        assertThat(demo.getName()).isEqualTo(demoName);
        assertThat(demo.getVisits()).isEqualTo(visits.size());
    }
}
